import uuid

from django.conf import settings
from django.db import models


class FileStatus(models.TextChoices):
    DRAFT = 'DRAFT'
    SUCCESS = 'SUCCESS'
    ERROR = 'ERROR'
    PROCESS = 'PROCESS'


class ImportFile(models.Model):
    Status = FileStatus

    id = models.UUIDField(primary_key=True, default=uuid.uuid4, editable=False)
    date = models.DateField()
    real_name = models.CharField(max_length=500)
    store_name = models.CharField(max_length=500)
    size = models.BigIntegerField()
    status = models.CharField(max_length=20, choices=FileStatus.choices)
    active = models.BooleanField()
    created_at = models.DateTimeField(auto_now_add=True)
    updated_at = models.DateTimeField(auto_now=True)
    deleted_at = models.DateTimeField(blank=True, null=True)
    created_by = models.UUIDField()
    updated_by = models.UUIDField(blank=True, null=True)
    deleted_by = models.UUIDField(blank=True, null=True)

    class Meta:
        app_label = settings.FILE_UPLOAD_APP_LABEL or 'file_upload'
        db_table = 'import_files'
        ordering = ('-created_at',)



from dataclasses import dataclass, field, asdict
from enum import Enum
from typing import Any

from .color import darken_color

ApexChartSeriesType = list[tuple[Any, ...]] | list[tuple[Any, Any]] | list[dict]


@dataclass(kw_only=True)
class ApexChartDataSeries:
    name: str = None
    color: str = None
    data: ApexChartSeriesType = field(default=list)


@dataclass(kw_only=True)
class ApexChartData:
    name: str
    color: str
    series: list[ApexChartDataSeries] = field(default_factory=list)
    columns: list = field(default_factory=list)

    def __post_init__(self):
        for index, series in enumerate(self.series):
            if series.name is None:
                series.name = self.name
            else:
                series.name = f'{self.name} ({series.name})'

            if series.color is None:
                series.color = darken_color(self.color, 1 - 0.1 * index)

    def add_series(self, data: ApexChartSeriesType, name=None, color=None):
        if name is None:
            name = self.name

        if color is None:
            color = self.color

        series = ApexChartDataSeries(data=data, name=name, color=color)
        self.series.append(series)
        return series


@dataclass(kw_only=True)
class ApexChart:
    headers: list[str] = field(default_factory=list)
    formats: list[str] = field(default_factory=list)
    data: list[ApexChartData] = field(default_factory=list)

    def add_data(self,
                 name: str,
                 color: str,
                 series: list[ApexChartSeriesType] = None,
                 columns: list = None):
        if series is None:
            series = []

        if columns is None:
            columns = []

        chart_data = ApexChartData(name=name, color=color, series=series, columns=columns)
        self.data.append(chart_data)
        return chart_data

    def asdict(self):
        return asdict(self)


class ValueFormats(str, Enum):
    KV_H = 'kV_h'
    MTV_H = 'MVt_h'
    KVTH = 'kVt*h'
    KVAR_H = 'kVar*h'
    KVR_H = 'kVr_h'
    KVT_H = 'kVt_h'
    SHT = 'sht'
    ED = 'ed'
    MLN_SUM = 'mln_sum'
    TONN = 'tonn'
    TN = 'tn'
    SUM = 'sum'
    M3_H = 'm3_h'
    M3 = 'm3'
    MT_H = 'mt_h'
    MLN_KVT_H = 'mln_kVt_h'
    MLN_M3 = 'mln_m3'
    T_C = '°C'



import mimetypes
from typing import BinaryIO

from django.core.files.uploadedfile import InMemoryUploadedFile
from minio import Minio


class MinioService:
    def __init__(self, endpoint, access_key, secret_key):
        self.client = Minio(
            endpoint=endpoint,
            access_key=access_key,
            secret_key=secret_key,
            secure=False
        )

    def fput_object(self, bucket_name, object_name, file_path):
        content_type, *_ = mimetypes.guess_type(file_path)
        return self.client.fput_object(bucket_name, object_name, file_path, content_type=content_type)

    def put_object(self, bucket_name, object_name, file: BinaryIO | InMemoryUploadedFile):
        content_type = "application/octet-stream"
        length = -1
        if hasattr(file, 'content_type'):
            content_type = file.content_type
        if hasattr(file, '__len__'):
            length = len(file)
        return self.client.put_object(bucket_name, object_name, file,
                                      length=length,
                                      content_type=content_type,
                                      part_size=10 * 1024 * 1024)

    def get_object(self, bucket_name, object_name):
        try:
            response = self.client.get_object(bucket_name, object_name)
            object_bytes = response.read()
        finally:
            response.close()
            response.release_conn()
        return object_bytes



from .file import ImportFileViewSet, FileDownloadViewSet, FileUploadViewSet, FileConfirmViewSet
from .file_upload_status import FileUploadStatusView

__all__ = [
    'ImportFileViewSet',
    'FileConfirmViewSet',
    'FileUploadViewSet',
    'FileDownloadViewSet',
    'FileUploadStatusView',
]



import random

random.seed(1)


def hex_to_rgb(hex_color):
    hex_color = hex_color.lstrip('#')
    return tuple(int(hex_color[i:i+2], 16) for i in (0, 2, 4))


def rgb_to_hex(rgb):
    return "#{:02x}{:02x}{:02x}".format(rgb[0], rgb[1], rgb[2])


def darken_color(hex_color, factor):
    if not (0 <= factor <= 1):
        raise ValueError("Factor must be between 0 and 1")

    r, g, b = hex_to_rgb(hex_color)
    darkened_rgb = (int(r * factor), int(g * factor), int(b * factor))
    return rgb_to_hex(darkened_rgb)


def generate_random_color():
    r = random.randint(0, 255)
    g = random.randint(0, 255)
    b = random.randint(0, 255)
    return "#{:02X}{:02X}{:02X}".format(r, g, b)
